from messagebus import TokenCreator, TokenPriority


MSGGRP_SLCTRL_PRODMGR = "9"

MSGGRP_SLCTRL_RUPTURE = "42"

TK_SLCTRL_PMGR_GET_TENDERS = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_PRODMGR,
    number="68",
)


TK_SLCTRL_PMGR_RUPTURE_UPDATE_ITEMS = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_RUPTURE,
    number="8",
)


def get_tokens():
    return (
        TK_SLCTRL_PMGR_GET_TENDERS,
    )
