from typing import (
    List,
    Optional,
)

from ._DeviceStatusHistory import DeviceStatusHistory


class DeviceStatus(object):
    def __init__(
        self,
        device_name,   # type: str
        enabled,       # type: bool
        history=None,  # type: Optional[List[DeviceStatusHistory]]
    ):
        # type: (...) -> None

        self.device_name = device_name
        self.enabled = enabled
        self.history = history if history is not None else []
