data = (
'jjwaels',    # 0x00
'jjwaelt',    # 0x01
'jjwaelp',    # 0x02
'jjwaelh',    # 0x03
'jjwaem',    # 0x04
'jjwaeb',    # 0x05
'jjwaebs',    # 0x06
'jjwaes',    # 0x07
'jjwaess',    # 0x08
'jjwaeng',    # 0x09
'jjwaej',    # 0x0a
'jjwaec',    # 0x0b
'jjwaek',    # 0x0c
'jjwaet',    # 0x0d
'jjwaep',    # 0x0e
'jjwaeh',    # 0x0f
'jjoe',    # 0x10
'jjoeg',    # 0x11
'jjoegg',    # 0x12
'jjoegs',    # 0x13
'jjoen',    # 0x14
'jjoenj',    # 0x15
'jjoenh',    # 0x16
'jjoed',    # 0x17
'jjoel',    # 0x18
'jjoelg',    # 0x19
'jjoelm',    # 0x1a
'jjoelb',    # 0x1b
'jjoels',    # 0x1c
'jjoelt',    # 0x1d
'jjoelp',    # 0x1e
'jjoelh',    # 0x1f
'jjoem',    # 0x20
'jjoeb',    # 0x21
'jjoebs',    # 0x22
'jjoes',    # 0x23
'jjoess',    # 0x24
'jjoeng',    # 0x25
'jjoej',    # 0x26
'jjoec',    # 0x27
'jjoek',    # 0x28
'jjoet',    # 0x29
'jjoep',    # 0x2a
'jjoeh',    # 0x2b
'jjyo',    # 0x2c
'jjyog',    # 0x2d
'jjyogg',    # 0x2e
'jjyogs',    # 0x2f
'jjyon',    # 0x30
'jjyonj',    # 0x31
'jjyonh',    # 0x32
'jjyod',    # 0x33
'jjyol',    # 0x34
'jjyolg',    # 0x35
'jjyolm',    # 0x36
'jjyolb',    # 0x37
'jjyols',    # 0x38
'jjyolt',    # 0x39
'jjyolp',    # 0x3a
'jjyolh',    # 0x3b
'jjyom',    # 0x3c
'jjyob',    # 0x3d
'jjyobs',    # 0x3e
'jjyos',    # 0x3f
'jjyoss',    # 0x40
'jjyong',    # 0x41
'jjyoj',    # 0x42
'jjyoc',    # 0x43
'jjyok',    # 0x44
'jjyot',    # 0x45
'jjyop',    # 0x46
'jjyoh',    # 0x47
'jju',    # 0x48
'jjug',    # 0x49
'jjugg',    # 0x4a
'jjugs',    # 0x4b
'jjun',    # 0x4c
'jjunj',    # 0x4d
'jjunh',    # 0x4e
'jjud',    # 0x4f
'jjul',    # 0x50
'jjulg',    # 0x51
'jjulm',    # 0x52
'jjulb',    # 0x53
'jjuls',    # 0x54
'jjult',    # 0x55
'jjulp',    # 0x56
'jjulh',    # 0x57
'jjum',    # 0x58
'jjub',    # 0x59
'jjubs',    # 0x5a
'jjus',    # 0x5b
'jjuss',    # 0x5c
'jjung',    # 0x5d
'jjuj',    # 0x5e
'jjuc',    # 0x5f
'jjuk',    # 0x60
'jjut',    # 0x61
'jjup',    # 0x62
'jjuh',    # 0x63
'jjweo',    # 0x64
'jjweog',    # 0x65
'jjweogg',    # 0x66
'jjweogs',    # 0x67
'jjweon',    # 0x68
'jjweonj',    # 0x69
'jjweonh',    # 0x6a
'jjweod',    # 0x6b
'jjweol',    # 0x6c
'jjweolg',    # 0x6d
'jjweolm',    # 0x6e
'jjweolb',    # 0x6f
'jjweols',    # 0x70
'jjweolt',    # 0x71
'jjweolp',    # 0x72
'jjweolh',    # 0x73
'jjweom',    # 0x74
'jjweob',    # 0x75
'jjweobs',    # 0x76
'jjweos',    # 0x77
'jjweoss',    # 0x78
'jjweong',    # 0x79
'jjweoj',    # 0x7a
'jjweoc',    # 0x7b
'jjweok',    # 0x7c
'jjweot',    # 0x7d
'jjweop',    # 0x7e
'jjweoh',    # 0x7f
'jjwe',    # 0x80
'jjweg',    # 0x81
'jjwegg',    # 0x82
'jjwegs',    # 0x83
'jjwen',    # 0x84
'jjwenj',    # 0x85
'jjwenh',    # 0x86
'jjwed',    # 0x87
'jjwel',    # 0x88
'jjwelg',    # 0x89
'jjwelm',    # 0x8a
'jjwelb',    # 0x8b
'jjwels',    # 0x8c
'jjwelt',    # 0x8d
'jjwelp',    # 0x8e
'jjwelh',    # 0x8f
'jjwem',    # 0x90
'jjweb',    # 0x91
'jjwebs',    # 0x92
'jjwes',    # 0x93
'jjwess',    # 0x94
'jjweng',    # 0x95
'jjwej',    # 0x96
'jjwec',    # 0x97
'jjwek',    # 0x98
'jjwet',    # 0x99
'jjwep',    # 0x9a
'jjweh',    # 0x9b
'jjwi',    # 0x9c
'jjwig',    # 0x9d
'jjwigg',    # 0x9e
'jjwigs',    # 0x9f
'jjwin',    # 0xa0
'jjwinj',    # 0xa1
'jjwinh',    # 0xa2
'jjwid',    # 0xa3
'jjwil',    # 0xa4
'jjwilg',    # 0xa5
'jjwilm',    # 0xa6
'jjwilb',    # 0xa7
'jjwils',    # 0xa8
'jjwilt',    # 0xa9
'jjwilp',    # 0xaa
'jjwilh',    # 0xab
'jjwim',    # 0xac
'jjwib',    # 0xad
'jjwibs',    # 0xae
'jjwis',    # 0xaf
'jjwiss',    # 0xb0
'jjwing',    # 0xb1
'jjwij',    # 0xb2
'jjwic',    # 0xb3
'jjwik',    # 0xb4
'jjwit',    # 0xb5
'jjwip',    # 0xb6
'jjwih',    # 0xb7
'jjyu',    # 0xb8
'jjyug',    # 0xb9
'jjyugg',    # 0xba
'jjyugs',    # 0xbb
'jjyun',    # 0xbc
'jjyunj',    # 0xbd
'jjyunh',    # 0xbe
'jjyud',    # 0xbf
'jjyul',    # 0xc0
'jjyulg',    # 0xc1
'jjyulm',    # 0xc2
'jjyulb',    # 0xc3
'jjyuls',    # 0xc4
'jjyult',    # 0xc5
'jjyulp',    # 0xc6
'jjyulh',    # 0xc7
'jjyum',    # 0xc8
'jjyub',    # 0xc9
'jjyubs',    # 0xca
'jjyus',    # 0xcb
'jjyuss',    # 0xcc
'jjyung',    # 0xcd
'jjyuj',    # 0xce
'jjyuc',    # 0xcf
'jjyuk',    # 0xd0
'jjyut',    # 0xd1
'jjyup',    # 0xd2
'jjyuh',    # 0xd3
'jjeu',    # 0xd4
'jjeug',    # 0xd5
'jjeugg',    # 0xd6
'jjeugs',    # 0xd7
'jjeun',    # 0xd8
'jjeunj',    # 0xd9
'jjeunh',    # 0xda
'jjeud',    # 0xdb
'jjeul',    # 0xdc
'jjeulg',    # 0xdd
'jjeulm',    # 0xde
'jjeulb',    # 0xdf
'jjeuls',    # 0xe0
'jjeult',    # 0xe1
'jjeulp',    # 0xe2
'jjeulh',    # 0xe3
'jjeum',    # 0xe4
'jjeub',    # 0xe5
'jjeubs',    # 0xe6
'jjeus',    # 0xe7
'jjeuss',    # 0xe8
'jjeung',    # 0xe9
'jjeuj',    # 0xea
'jjeuc',    # 0xeb
'jjeuk',    # 0xec
'jjeut',    # 0xed
'jjeup',    # 0xee
'jjeuh',    # 0xef
'jjyi',    # 0xf0
'jjyig',    # 0xf1
'jjyigg',    # 0xf2
'jjyigs',    # 0xf3
'jjyin',    # 0xf4
'jjyinj',    # 0xf5
'jjyinh',    # 0xf6
'jjyid',    # 0xf7
'jjyil',    # 0xf8
'jjyilg',    # 0xf9
'jjyilm',    # 0xfa
'jjyilb',    # 0xfb
'jjyils',    # 0xfc
'jjyilt',    # 0xfd
'jjyilp',    # 0xfe
'jjyilh',    # 0xff
)
