class VariableNotGeneratedByBuilder(Exception):

    def __init__(
        self,
        variable_name: str,
    ) -> None:

        self.variable_name = variable_name

    def __str__(
        self
    ) -> str:

        return f"Error variable_name: {self.variable_name}"
